unit Main;
{$I ..\PDXRBLD.INC}

interface

uses
  SysUtils, WinTypes, WinProcs, Ver, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, TabNotBk, ExtCtrls, StdCtrls, Gauges, RKSIni,
  dbiTypes,dbiProcs,dbiErrs, DB, FileCtrl, ShellAPI,
  VarTbl,
  TUtility, PxHeader, Rbld1, Translat;

const
  WM_RUNCMDLINE = WM_USER + 2;

type
  TMainForm = class(TForm)
    PageControl1: TTabbedNotebook;
    Panel1: TPanel;
    RadioAlias: TRadioGroup;
    rgRebuildScope: TRadioGroup;
    AliasCombo: TComboBox;
    Label1: TLabel;
    DirectoryCombo: TComboBox;
    Label2: TLabel;
    ButtonBrowse: TButton;
    tableCombo: TComboBox;
    Label3: TLabel;
    CheckDoPack: TCheckBox;
    CheckDoLog: TCheckBox;
    Bevel1: TBevel;
    Image1: TImage;
    ButtonStart: TButton;
    ButtonStop: TButton;
    ButtonClose: TButton;
    ButtonViewLog: TButton;
    LabelInfo: TLabel;
    LabelTablename: TLabel;
    LabelWhatDoing: TLabel;
    LabelDetails: TLabel;
    ProgressDatabase: TGauge;
    ProgressTable: TGauge;
    ListView1: TListBox;
    Header1: THeader;
    OpenDialog1: TOpenDialog;
    CheckSkipVerify: TCheckBox;
    procedure ListView1DrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Header1Sized(Sender: TObject; ASection, AWidth: Integer);
    procedure ButtonStartClick(Sender: TObject);
    procedure ButtonStopClick(Sender: TObject);
    procedure ButtonCloseClick(Sender: TObject);
    procedure ButtonViewLogClick(Sender: TObject);
    procedure ButtonBrowseClick(Sender: TObject);
    procedure RadioAliasClick(Sender: TObject);
    procedure tableComboEnter(Sender: TObject);
    procedure AliasComboChange(Sender: TObject);
    procedure DirectoryComboChange(Sender: TObject);
    procedure rgRebuildScopeClick(Sender: TObject);
  private
    { Private declarations }
    msgMessages: tStringList;
    procedure AddListViewMessage(const atbl,amessage: string; aProcess: TRebuildProcess);
    procedure AdjustListBox(AdustLeftColumn: Boolean);
  public
    { Public declarations }
    procedure WMRunCmdLine(Var Message:tMessage); message wm_runcmdline;
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation
Uses LowStuff;

{$R *.DFM}
{$I ..\main.inc}

{helpers 16 bit}
Procedure CellTextRect(Canvas: tCanvas;
                     HAlign,VAlign: integer;   {-1: left align,0: center,1: right align}
                     CellR: tRect; { Rect to align to }
                     ClipR: tRect; { Rect to clip to }
                     Text: string);
Var SavAlign,NewAlign : SmallInt;
    RefX,RefY: integer;
    SavBStyle: tBrushStyle;
    tHght: Integer;
begin
  if Canvas = NIL then exit;
  with Canvas do begin
    SavAlign := GetTextAlign(Handle);
    Case HAlign of
    -1: begin
          with CellR do RefX := left;
          NewAlign := TA_Left;
        end;
     0: begin
          with CellR do RefX := (left + Right) div 2;
          NewAlign := TA_Center;
        end;
    else begin
            with CellR do RefX := Right;
            NewAlign := TA_Right;
         end;
    end;
    Case VAlign of
    -1: begin
          with CellR do RefY := Top;
          NewAlign := NewAlign OR TA_Top;
        end;
     0: begin
          tHght := TextHeight(Text);
          with CellR do RefY := Top + ((Bottom - Top - tHght) div 2);
          NewAlign := NewAlign OR TA_Top;
        end;
    else begin
           with CellR do RefY := Bottom;
           NewAlign := NewAlign OR TA_Bottom;
         end;
    end;
    NewAlign := SetTextAlign(Handle,NewAlign OR TA_NOUPDATECP);
    SavBStyle := Canvas.Brush.Style;
    Canvas.Brush.Style := bsClear;
    Canvas.TextRect(ClipR, RefX, RefY,Text);
    SetTextAlign(Handle,SavAlign);
    Canvas.Brush.Style := SavBStyle;
  end;
end;

{ tMainForm }

procedure TMainForm.AdjustListBox(AdustLeftColumn: Boolean);
var
  i,x,
  ltextw,
  rtextw: integer;
begin
  rtextw := 0;
  for i := 0 to msgMessages.Count - 1 do begin
    x := ListView1.canvas.TextWidth(msgMessages[i]);
    if x > rtextw then rtextw := x;
  end;
  inc(rtextw,10);
  if AdustLeftColumn then begin
    ltextw := 0;
    for i := 0 to ListView1.Items.Count - 1 do begin
      x := ListView1.canvas.TextWidth(ListView1.Items[i]);
      if x > ltextw then ltextw := x;
    end;
    inc(ltextw,10);
    Header1.SectionWidth[0] := ltextw;
  end else ltextw := Header1.SectionWidth[0];
  SendMessage(ListView1.handle, LB_SETHORIZONTALEXTENT,ltextw + rtextw,0);
  Header1.SectionWidth[1] := rtextw;
end;

procedure TMainForm.AddListViewMessage(const atbl,amessage: string; aProcess: TRebuildProcess);
begin
  ListView1.Items.Add(aTbl);
  msgMessages.add(amessage);
  tblToList := ''; { table name will be written only once to the listview}
  if not pagecontrol1.controls[1].Visible then begin
    With ListView1 do begin
      Header1.Sections[0] := LoadStr(dfm_ListView1Column0Caption);
      Header1.Sections[1] := LoadStr(dfm_ListView1Column1Caption);
    end;
    pagecontrol1.controls[1].Visible := True;
  end;
  AdjustListBox(True);
  if aProcess <> cpPack then Inc(LastTblErrCount);
  if LogToFile then
    if CompareText(ExtractFileExt(LastTbl),'.DB') = 0 then
      Writeln(LogFile,LoadStr(msg_TblnamePrefix),
                      Padright(LastTbl,TblnamePadLen),
                      LoadStr(msg_MsgSeparator),
                      amessage)
    else
      Writeln(LogFile,LoadStr(msg_IndexnamePrefix),
                      Padright(LastTbl,TblnamePadLen),
                      LoadStr(msg_MsgSeparator),
                      amessage)
end;

procedure TMainForm.ListView1DrawItem(Control: TWinControl; Index: Integer;
  Rect: TRect; State: TOwnerDrawState);
Var R:tRect;
    s: string;
begin
  if odFocused in State then exit;
  s := ListView1.Items[Index];
  ListView1.Canvas.FillRect(Rect);
  R := Rect;
  R.Right := R.Left + Header1.SectionWidth[0];
  inc(R.left,5);
  CellTextRect(ListView1.Canvas,-1,0,R,R,s);
  R.Left  := R.Right;
  inc(R.left,5);
  s := msgMessages[Index];
  R.Right := R.Left + ListView1.Canvas.TextWidth(s);
  CellTextRect(ListView1.Canvas,-1,0,R,R,s);
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  TMainFormFormCreate(Sender);
  msgMessages:= tStringList.Create;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  msgMessages.Free;
  TMainFormFormDestroy(Sender);
end;

procedure TMainForm.Header1Sized(Sender: TObject; ASection,
  AWidth: Integer);
begin
  AdjustListBox(False);
  ListView1.Invalidate;
end;

procedure TMainForm.WMRunCmdLine(Var Message:tMessage);
begin
  TMainFormWMRunCmdLine(Message);
end;

procedure TMainForm.ButtonStartClick(Sender: TObject);
begin
  TMainFormButtonStartClick(Sender);   
end;

procedure TMainForm.ButtonStopClick(Sender: TObject);
begin
  TMainFormButtonStopClick(Sender);
end;

procedure TMainForm.ButtonCloseClick(Sender: TObject);
begin
  TMainFormButtonCloseClick(Sender);
end;

procedure TMainForm.ButtonViewLogClick(Sender: TObject);
begin
  TMainFormButtonViewLogClick(Sender);
end;

procedure TMainForm.ButtonBrowseClick(Sender: TObject);
begin
  TMainFormButtonBrowseClick(Sender);
end;

procedure TMainForm.RadioAliasClick(Sender: TObject);
begin
  TMainFormRadioAliasClick(Sender);
end;

procedure TMainForm.tableComboEnter(Sender: TObject);
begin
  TMainFormtableComboEnter(Sender);
end;

procedure TMainForm.AliasComboChange(Sender: TObject);
begin
  TMainFormAliasComboChange(Sender);
end;

procedure TMainForm.DirectoryComboChange(Sender: TObject);
begin
  TMainFormDirectoryComboChange(Sender);
end;

procedure TMainForm.rgRebuildScopeClick(Sender: TObject);
begin
  TMainFormrgRebuildScopeClick(Sender);
end;

initialization
  Initialize;

end.
